pub enum RegisterOperation<T> {
Read(Option<T>),
Write(T),
}Expand description
An operation for a register.
Variants
Read(Option<T>)
Read a value of type T from the register.
If the return value of the operation is not-yet-known, then this can be
represented as Read(None).
Write(T)
Write a value of type T to the register.